<?php

namespace Lumav\EveryPay\Block\System\Config;

class Button extends \Magento\Config\Block\System\Config\Form\Field
{
    /**
     * @var string
     */
    protected $_template = 'Lumav_EveryPay::system/config/button.phtml';

    /**
     * @var \Magento\Backend\Model\Url
     */
    protected $url;
    
    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * @var \Magento\Backend\Block\Template\Context $context
     * @var array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Model\Url $url,
        \Magento\Framework\App\Request\Http $request,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->url = $url;
        $this->request = $request;
    }

    /**
     * @var \Magento\Framework\Data\Form\Element\AbstractElement 
     */
    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Creates custom button html
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock(
            'Magento\Backend\Block\Widget\Button'
        )->setData(
            [
                'id' => 'update_payment_methods_btn',
                'label' => __('Update'),
                'onclick' => 'updateMethods(this)'
            ]
        );
        return $button->toHtml();
    }

    /**
     * Returns popup title 
     *
     * @return string
     */
    public function getMessageTitle()
    {
        return __('Update Message');
    }

    /**
     * Returns success message
     * 
     * @return string
     */
    public function getSuccessMessage()
    {
        return __('Payment methods were successfully updated.');
    }

    /**
     * Returns error message
     * 
     * @return string
     */
    public function getErrorMessage()
    {
        return __('An error occurred while updating payment methods.');
    }

    /**
     * Returns update controller's url
     *
     * @return string
     */
    public function getUpdateUrl()
    {
        $storeId = $this->request->getParam('store');
        return $this->url->getUrl('everypay/methods/update', array('store' => $storeId));
    }
}