<?php

namespace Lumav\EveryPay\Controller\Customer;

use Magento\Framework\Controller\ResultFactory;

/**
 * Class Landing
 */
class Landing extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Lumav\EveryPay\Helper\Validator
     */
    protected $validator;

    /**
     * @var \Lumav\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Lumav\EveryPay\Helper\Validator $validator
     * @param \Lumav\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\Registry $registry,
        \Lumav\EveryPay\Helper\Validator $validator,
        \Lumav\EveryPay\Logger\Logger $logger
    ) {
        $this->registry = $registry;
        $this->validator = $validator;
        $this->logger = $logger;
        parent::__construct($context);
    }

    /**
     * {@inheritDoc}
     */
    public function execute()
    {
        if ($this->getRequest()->isGet()) {
            $reference = $this->getRequest()->getParam('order_reference');
            $this->logger->info(sprintf(__('Customer has returned to shop with order reference %s'), $reference));
            $result = $this->validator->isOrderPaid($reference);
            
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            if (is_array($result)) {
                foreach ($result as $err) {
                    $this->logger->info(__($err));
                }
                return $resultRedirect->setPath('checkout/onepage/failure');
            }
            
            if ($result) {
                return $resultRedirect->setPath('checkout/onepage/success');
            } else {
                if ($this->registry->registry('order_reference')) {
                    $this->registry->unregister('order_reference');
                }
                $this->registry->register('order_reference', $reference);
                
                $resultPage = $this->resultFactory->create(ResultFactory::TYPE_PAGE);
                return $resultPage;
            }
        }
    }
}