<?php

namespace Lumav\EveryPay\Helper;

class Validator extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * Taken from https://github.com/UnifiedPaymentSolutions/everypay-integration/blob/master/php/EveryPay.php::157
     * (as of 25.01.2017)
     */
    const RESPONSE_AGE = 600;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $orderInterface;

    /**
     * @var \Lumav\EveryPay\Model\Config
     */
    protected $config;

    /**
     * @var array
     */
    protected $errors = [];

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Sales\Api\Data\OrderInterface $orderInterface
     * @param \Lumav\EveryPay\Model\Config $config
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Sales\Api\Data\OrderInterface $orderInterface,
        \Lumav\EveryPay\Model\Config $config
    ) {
        $this->orderInterface = $orderInterface;
        $this->config = $config;
        parent::__construct($context);
    }

    /**
     * @param array $data
     * @return bool|array
     */
    public function validate(array $data)
    {
        $this->errors = [];

        $this
            ->validateTimestamp($data)
            ->validateOrderReference($data)
            ->validateUsername($data);

        if (!empty($this->errors)) {
            return $this->errors;
        }

        return true;
    }

    /**
     * @param string $orderReference
     * @return bool|array
     */
    public function isOrderPaid($orderReference)
    {
        $this->errors = [];

        if (!$orderReference) {
            $this->errors[] = __('No order reference given');
        }

        $this->validateOrderReference(['order_reference' => $orderReference]);
        if (!empty($this->errors)) {
            return $this->errors;
        }
        $order = $this->orderInterface->loadByIncrementId($orderReference);
        return $order->hasInvoices();
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateUsername(array $data)
    {
        if (!isset($data['api_username']) || $data['api_username'] !== $this->config->getApiUser()) {
            $this->errors[] = __('Response username is invalid');
        }

        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateOrderReference(array $data)
    {
        try {
            $this->orderInterface->loadByIncrementId($data['order_reference']);
        } catch (\Exception $e) {
            $this->errors[] = __('Order does not exist');
        }

        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateTimestamp(array $data)
    {
        $now = time();
        $createdAt = isset($data['payment_created_at']) ? strtotime($data['payment_created_at']) : null;

        if (!$createdAt || ($createdAt > $now) || ($createdAt < ($now - self::RESPONSE_AGE))) {
            $this->errors[] = __('Response is outdated');
        }

        return $this;
    }
}