<?php

namespace Lumav\EveryPay\Model;

use Lumav\EveryPay\Model\Adminhtml\Source\Environment;
use Magento\Framework\App\Config\Storage\WriterInterface;

class Config
{
    const XML_PATH_FORM_TYPE = 'form_type';
    const XML_PATH_SKIN_NAME = 'skin_name';
    const XML_PATH_ENVIRONMENT = 'environment';
    const XML_PATH_API_USER_PRODUCTION = 'api_user_production';
    const XML_PATH_API_USER_TEST = 'api_user_test';
    const XML_PATH_API_PASSWORD_PRODUCTION = 'api_password_production';
    const XML_PATH_API_PASSWORD_TEST = 'api_password_test';
    const XML_PATH_PROCESSING_ACCOUNT = 'processing_account';
    const XML_PATH_CC_TOKEN_ENABLED = 'cc_token_enabled';
    const XML_PATH_TOKEN_SECURITY = 'token_security';
    const XML_PATH_TEST_API_ENDPOINT = 'test_api_endpoint';
    const XML_PATH_PRODUCTION_API_ENDPOINT = 'production_api_endpoint';
    const XML_PATH_TEST_ONE_OFF_API_ENDPOINT = 'test_one_off_api_endpoint';
    const XML_PATH_PRODUCTION_ONE_OFF_API_ENDPOINT = 'production_one_off_api_endpoint';
    const XML_PATH_DEFAULT_LOCALE = 'default_locale';
    const XML_PATH_SUPPORTED_LOCALES = 'supported_locales';
    const XML_PATH_DEFAULT_COUNTRY = 'default_country';
    const XML_PATH_SUPPORTED_COUNTRIES = 'supported_countries';
    const XML_PATH_CURRENCIES_ALLOWED = 'currencies_allowed';
    const XML_PATH_DISPLAY_LOGOS = 'display_logos';
    const XML_PATH_PAYMENT_METHODS = 'methods';
    const XML_PATH_METHODS_ORDER = 'methods_order';
    const XML_PATH_APPLE_BUTTON = 'apple_button';
    const XML_PATH_GOOGLE_BUTTON = 'google_button';

    const NEW_CARD_ID = -1;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var WriterInterface
     */
    protected $configWriter;
    
    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $appState;

    /**
     * @param \Magento\Store\Model\StoreManagerInterface $storeManagerInterface
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigInterface
     * @param WriterInterface $configWriter
     */
    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigInterface,
        WriterInterface $configWriter,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Framework\App\State $appState
    ) {
        $this->storeManager = $storeManagerInterface;
        $this->scopeConfig = $scopeConfigInterface;
        $this->configWriter = $configWriter;
        $this->request = $request;
        $this->appState = $appState;
    }

    /**
     * Retrieve information from payment configuration
     *
     * @param string $field
     * @param int|string|null|\Magento\Store\Model\Store $storeId
     *
     * @return mixed
     */
    public function getConfigData($field, $storeId = null)
    {
        if (null === $storeId) {
            $storeId = $this->storeManager->getStore()->getId();
        }
        $path = 'payment/everypay/' . $field;
        return $this->scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORES, $storeId);
    }

    /**
     * Set payment configuration value
     *
     * @param string $field
     * @param mixed $value
     * @param int|string|null|\Magento\Store\Model\Store $storeId
     * @return void
     */
    public function setConfigData($field, $value, $storeId = null)
    {
        $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORES;
        if (null === $storeId) {
            if($this->isAdmin()) {
                $storeId = $this->request->getParam('store') ? : 0;
            } else {
                $storeId = $this->storeManager->getStore()->getId();
            }
        }
        if($storeId == 0) {
            $scope = \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT;
        }
        $path = 'payment/everypay/' . $field;
        $this->configWriter->save($path, $value, $scope, $storeId);
    }

    /**
     * @return string
     */
    public function getFormType()
    {
        return trim($this->getConfigData(Config::XML_PATH_FORM_TYPE));
    }

    /**
     * @return string
     */
    public function getSkinName()
    {
        $skinName = trim($this->getConfigData(Config::XML_PATH_SKIN_NAME));

        if (empty($skinName)) {
            $skinName = 'default';
        }

        return $skinName;
    }

    /**
     * @return null|string
     */
    public function getApiUser()
    {
        $environment = $this->getEnvironment();

        if ($environment == Environment::ENVIRONMENT_PRODUCTION) {
            return $this->getApiUserProduction();
        }

        if ($environment == Environment::ENVIRONMENT_TEST) {
            return $this->getApiUserTest();
        }

        return null;
    }

    /**
     * @return null|string
     */
    public function getApiPassword()
    {
        $environment = $this->getEnvironment();

        if ($environment == Environment::ENVIRONMENT_PRODUCTION) {
            return $this->getApiPasswordProduction();
        }

        if ($environment == Environment::ENVIRONMENT_TEST) {
            return $this->getApiPasswordTest();
        }

        return null;
    }

    /**
     * @return string
     */
    public function getEnvironment()
    {
        return $this->getConfigData(Config::XML_PATH_ENVIRONMENT);
    }

    /**
     * @return string
     */
    public function getApiUserProduction()
    {
        return trim($this->getConfigData(Config::XML_PATH_API_USER_PRODUCTION));
    }

    /**
     * @return string
     */
    public function getApiUserTest()
    {
        return trim($this->getConfigData(Config::XML_PATH_API_USER_TEST));
    }

    /**
     * @return string
     */
    public function getApiPasswordProduction()
    {
        return trim($this->getConfigData(Config::XML_PATH_API_PASSWORD_PRODUCTION));
    }

    /**
     * @return string
     */
    public function getApiPasswordTest()
    {
        return trim($this->getConfigData(Config::XML_PATH_API_PASSWORD_TEST));
    }

    /**
     * @return string
     */
    public function getProcessingAccount()
    {
        return trim($this->getConfigData(Config::XML_PATH_PROCESSING_ACCOUNT));
    }

    /**
     * @return boolean
     */
    public function getCcTokenEnabled()
    {
        return $this->getConfigData(Config::XML_PATH_CC_TOKEN_ENABLED) == 1;
    }

    /**
     * @return string
     */
    public function getTokenSecurity()
    {
        return $this->getConfigData(Config::XML_PATH_TOKEN_SECURITY);
    }

    /**
     * @return string
     */
    public function getTestApiEndpoint()
    {
        file_put_contents(BP . '/var/log/tst.log', $this->getConfigData(Config::XML_PATH_TEST_API_ENDPOINT), FILE_APPEND);
        return $this->getConfigData(Config::XML_PATH_TEST_API_ENDPOINT);
    }

    /**
     * @return string
     */
    public function getProductionApiEndpoint()
    {
        return $this->getConfigData(Config::XML_PATH_PRODUCTION_API_ENDPOINT);
    }

    /**
     * @return null|string
     */
    public function getApiEndpoint()
    {
        $environment = $this->getEnvironment();

        if ($environment == Environment::ENVIRONMENT_PRODUCTION) {
            return $this->getProductionApiEndpoint();
        }

        if ($environment == Environment::ENVIRONMENT_TEST) {
            return $this->getTestApiEndpoint();
        }

        return null;
    }
    
    /**
     * @return string
     */
    public function getTestOneOffApiEndpoint()
    {
        file_put_contents(BP . '/var/log/tst.log', $this->getConfigData(Config::XML_PATH_TEST_ONE_OFF_API_ENDPOINT), FILE_APPEND);
        return $this->getConfigData(Config::XML_PATH_TEST_ONE_OFF_API_ENDPOINT);
    }

    /**
     * @return string
     */
    public function getProductionOneOffApiEndpoint()
    {
        return $this->getConfigData(Config::XML_PATH_PRODUCTION_ONE_OFF_API_ENDPOINT);
    }

    /**
     * @return null|string
     */
    public function getOneOffApiEndpoint()
    {
        $environment = $this->getEnvironment();

        if ($environment == Environment::ENVIRONMENT_PRODUCTION) {
            return $this->getProductionOneOffApiEndpoint();
        }

        if ($environment == Environment::ENVIRONMENT_TEST) {
            return $this->getTestOneOffApiEndpoint();
        }

        return null;
    }

    /**
     * @return string
     */
    public function getDefaultLocale()
    {
        return $this->getConfigData(Config::XML_PATH_DEFAULT_LOCALE);
    }

    /**
     * @return array
     */
    public function getSupportedLocales()
    {
        return explode(',', $this->getConfigData(Config::XML_PATH_SUPPORTED_LOCALES));
    }

    /**
     * @return string
     */
    public function getDefaultCountry()
    {
        return $this->getConfigData(Config::XML_PATH_DEFAULT_COUNTRY);
    }

    /**
     * @return array
     */
    public function getSupportedCountries()
    {
        return explode(',', $this->getConfigData(Config::XML_PATH_SUPPORTED_COUNTRIES));
    }

    /**
     * @param array $countries
     * @return void
     */
    public function setSupportedCountries($countries)
    {
        if (count($countries)) {
            $this->setConfigData(Config::XML_PATH_SUPPORTED_COUNTRIES, implode(',', $countries));
        }
    }

    /**
     * @return array
     */
    public function getAllowedCurrencies()
    {
        return array_map('trim', explode(',', $this->getConfigData('currencies_allowed')));
    }

    /**
     * @return bool
     */
    public function getDisplayLogos()
    {
        return $this->getConfigData(Config::XML_PATH_DISPLAY_LOGOS) == 1;
    }

    /**
     * @param string $paymentMethods
     * @return void
     */
    public function setPaymentMethods($paymentMethods)
    {
        $this->setConfigData(Config::XML_PATH_PAYMENT_METHODS, $paymentMethods);
    }

    /**
     * @return array
     */
    public function getPaymentMethods()
    {
        $methods = $this->getConfigData(Config::XML_PATH_PAYMENT_METHODS);
        return empty($methods) ? [] : json_decode($methods, true);
    }

    /**
     * @return array
     */
    public function getMethodsOrder()
    {
        $order = $this->getConfigData(Config::XML_PATH_METHODS_ORDER);
        return explode(',', $order);
    }
    
    /**
     * @return bool
     */
    protected function isAdmin()
    {
        $areaCode = $this->appState->getAreaCode();
        return $areaCode == \Magento\Framework\App\Area::AREA_ADMINHTML;
    }

    /**
     * @return bool
     */
    public function isApplePayAvailable()
    {
        $paymentMethods = $this->getPaymentMethods();
        if(isset($paymentMethods['cards'])) {
            foreach($paymentMethods['cards'] as $card) {
                if((!empty($card['applepay_available']) || !empty($card['applepay_custom_available'])) && !empty($card['applepay_merchant_identifier'])) {
                    return true;
                }
            }
        }
        
        return false;
    }

    /**
     * @return bool
     */
    public function getApplePayMerchantId()
    {
        $paymentMethods = $this->getPaymentMethods();
        if(isset($paymentMethods['cards'])) {
            foreach($paymentMethods['cards'] as $card) {
                if(!empty($card['applepay_merchant_identifier'])) {
                    return $card['applepay_merchant_identifier'];
                }
            }
        }
        
        return false;
    }
    
    /**
     * @return string
     */
    public function getAppleButton()
    {
        return $this->getConfigData(Config::XML_PATH_APPLE_BUTTON);
    }

    /**
     * @return bool
     */
    public function isGooglePayAvailable()
    {
        $paymentMethods = $this->getPaymentMethods();
        if(isset($paymentMethods['cards'])) {
            foreach($paymentMethods['cards'] as $card) {
                if((!empty($card['googlepay_available']) || !empty($card['googlepay_custom_available'])) && !empty($card['googlepay_merchant_identifier'])) {
                    return true;
                }
            }
        }
        
        return false;
    }

    /**
     * @return string|bool
     */
    public function getGooglePayMerchantId()
    {
        $paymentMethods = $this->getPaymentMethods();
        if(isset($paymentMethods['cards'])) {
            foreach($paymentMethods['cards'] as $card) {
                if(!empty($card['googlepay_merchant_identifier'])) {
                    return $card['googlepay_merchant_identifier'];
                }
            }
        }
        
        return false;
    }
    
    /**
     * @return string
     */
    public function getGoogleButton()
    {
        return $this->getConfigData(Config::XML_PATH_GOOGLE_BUTTON);
    }
}
