<?php

namespace Lumav\EveryPay\Model\Config\Backend;

class MethodsOrder extends \Magento\Framework\App\Config\Value
{
    /**
     * @var array
     */
    protected $methodLabels = [
        'card' => 'Card',
        'banks' => 'Banks',
        'alt' => 'Alternative',
        'apple' => 'Apple Pay',
        'google' => 'Google Pay'
    ];

    /**
     * Processing object after load data
     *
     * @return void
     */
    protected function _afterLoad()
    {
        $value = $this->getValue();
        if (!is_array($value)) {
            try {
                $codes = empty($value) ? $this->getMethodTypes() : $this->addMissingTypes($this->removeUnknownTypes(explode(',', $value)));
                $this->setValue($this->buildGridElements($codes));
            } catch (\Exception $e) {
                $this->_logger->critical(
                    sprintf(
                        'Failed to unserialize %s config value. The error is: %s',
                        $this->getPath(),
                        $e->getMessage()
                    )
                );
                $this->setValue(false);
            }
        }
    }

    /**
     * Build grid elements form codes
     *
     * @param array $codes
     * @return array
     */
    private function buildGridElements($codes)
    {
        $elements = [];
        foreach($codes as $code) {
            $elements[$code] = ['label' => __($this->methodLabels[$code])];
        }
        return $elements;
    }

    /**
     * Processing object before save data
     *
     * @return $this
     */
    public function beforeSave()
    {
        if (is_array($this->getValue())) {
            $values = $this->getValue();
            unset($values['__empty']);
            $codes = array_keys($values);
            // Get only correct codes
            $codes = $this->removeUnknownTypes($codes);
            $missingMethods = array_diff($this->getMethodTypes(), $codes);
            // Re-add missing codes
            $codes = array_merge($codes, $missingMethods);
            $this->setValue(implode(',', $codes));
        }
        parent::beforeSave();
        return $this;
    }

    /**
     * Remove unknown types.
     * 
     * @param array $codes
     * @return array
     */
    private function removeUnknownTypes($codes)
    {
        return array_intersect($codes, $this->getMethodTypes());
    }

    /**
     * Add missing types.
     * 
     * @param array $codes
     * @return array
     */
    private function addMissingTypes($codes)
    {
        foreach($this->getMethodTypes() as $type) {
            if(!in_array($type, $codes)) {
                $codes[] = $type;
            }
        }
        return $codes;
    }

    /**
     * Get default method types.
     * 
     * @return array
     */
    private function getMethodTypes()
    {
        return array_keys($this->methodLabels);
    }
}