<?php

namespace Lumav\EveryPay\Model;

use Lumav\EveryPay\Model\Gateway;

class EveryPayConfigProvider implements \Magento\Checkout\Model\ConfigProviderInterface
{
    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Framework\View\Asset\Repository
     */
    protected $assetRepository;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $url;

    /**
     * @var \Magento\Customer\Model\Url
     */
    protected $customerUrl;

    /**
     * @var Token\TokenManagement
     */
    protected $tokenManager;

    /**
     * @var \Magento\Directory\Model\Config\Source\Country
     */
    protected $countryList;

    /**
     * @var Config
     */
    protected $config;

    /**
     * EveryPayConfigProvider constructor.
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\View\Asset\Repository $assetRepository
     * @param \Magento\Framework\UrlInterface $url
     * @param \Magento\Customer\Model\Url $customerUrl
     * @param Token\TokenManagement $tokenManager
     * @param \Magento\Directory\Model\Config\Source\Country $countryList
     * @param Config $config
     */
    public function __construct(
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\View\Asset\Repository $assetRepository,
        \Magento\Framework\UrlInterface $url,
        \Magento\Customer\Model\Url $customerUrl,
        \Lumav\EveryPay\Model\Token\TokenManagement $tokenManager,
        \Magento\Directory\Model\Config\Source\Country $countryList,
        Config $config
    ) {
        $this->customerSession = $customerSession;
        $this->assetRepository = $assetRepository;
        $this->url = $url;
        $this->customerUrl = $customerUrl;
        $this->tokenManager = $tokenManager;
        $this->countryList = $countryList;
        $this->config = $config;
    }

    /**
     * @return array
     */
    public function getConfig()
    {

        $logos = $this->getLogos();
        return [
            'payment' => [
                'everypay' => [
                    'isIFrame' =>
                        $this->config->getFormType() ===
                        \Lumav\EveryPay\Model\Adminhtml\Source\FormType::FORM_TYPE_IFRAME,
                    'customer' => [
                            'cards' => $this->getCards(),
                            'isLoggedIn' => $this->customerSession->isLoggedIn(),
                    ],
                    'customerUrl' => $this->customerUrl->getLoginUrl(),
                    'isTokenEnabled' => $this->config->getCcTokenEnabled(),
                    'deleteUrl' => $this->url->getUrl('everypay/token/delete'),
                    'logos' => [$logos['visa'], $logos['master_card']], //a hack for ko.js
                    'methods' => $this->config->getPaymentMethods(),
                    'titles' => $this->getTitles(),
                    'countries' => $this->getCountries(),
                    'defaultCountry' => $this->config->getDefaultCountry(),
                    'methodsOrder' => $this->config->getMethodsOrder(),
                    'applepay_available' => $this->config->isApplePayAvailable(),
                    'appleButton' => $this->config->getAppleButton(),
                    'googlepay_available' => $this->config->isGooglePayAvailable(),
                    'googleButton' => $this->config->getGoogleButton()
                ]
            ]
        ];
    }

    /**
     * @return array
     */
    protected function getTitles()
    {
        return [
            Gateway::CARDS => $this->config->getConfigData('title_card'),
            Gateway::BANKS => $this->config->getConfigData('title_bank'),
            Gateway::ALTERNATIVE => $this->config->getConfigData('title_alt'),
            Gateway::APPLE => $this->config->getConfigData('title_apple'),
            Gateway::GOOGLE => $this->config->getConfigData('title_google')
        ];
    }

    /**
     * @return array
     */
    protected function getCountries()
    {
        $data = [];
        $codes = $this->config->getSupportedCountries();
        foreach ($this->countryList->toOptionArray() as $countryData) {
            if (isset($countryData['value']) && in_array($countryData['value'], $codes)) {
                $data[] = $countryData;
            }
        }
        return $data;
    }

    /**
     * @return array
     */
    protected function getLogos()
    {
        $visa = [
            'image' => $this->assetRepository->getUrl(
                sprintf("Lumav_EveryPay::images/visa.svg")
            ),
            'type' => 'visa',
        ];

        $mastercard = [
            'image' => $this->assetRepository->getUrl(
                sprintf("Lumav_EveryPay::images/master-card.svg")
            ),
            'type' => 'master-card',
        ];

        return [
            'visa' => $visa,
            'master_card' => $mastercard
        ];
    }

    /**
     * @return array
     */
    protected function getCards()
    {
        $cards = [];
        $logos = $this->getLogos();
        foreach ($this->tokenManager->getListByCustomerSession() as $token) {
            $cards[] = [
                'id' => $token['id'],
                'label' =>
                    sprintf("**** **** **** %s (%s %s/%s)",
                        $token['cc_last_four_digits'],
                        __('expires'),
                        $token['cc_month'],
                        $token['cc_year']
                    ),
                'image' => $logos[$token['cc_type']]['image'],
                'type' => str_replace('_', '-', $token['cc_type']),
            ];
        }

        if (!empty($cards)) {
            $newCard = [
                'id' => Config::NEW_CARD_ID,
                'label' => __('New card'),
                'image' => '',
                'type' => 'new-card'
            ];

            $cards[] = $newCard;
        }

        return $cards;
    }
}