<?php

namespace Lumav\EveryPay\Model\Form;

use Magento\Framework\Exception\LocalizedException;

class ApiData extends AbstractData
{
    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;

    /**
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Lumav\EveryPay\Model\Config $config
     */
    public function __construct(
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Lumav\EveryPay\Model\Config $config
    ) {
        $this->urlBuilder = $urlBuilder;

        parent::__construct($orderRepository, $config);
    }

    /**
     * @param int $orderId
     * @return array
     * @throws LocalizedException
     */
    public function getFields($orderId)
    {
        $data = [
            'api_username' => $this->config->getApiUser(),
            'account_name' => $this->config->getProcessingAccount(),
            'nonce' => $this->generateNonce(),
            'customer_url' => $this->urlBuilder->getRouteUrl('everypay/customer/landing'),
            'callback_url' => $this->urlBuilder->getRouteUrl('everypay/payment/status')
        ];

        foreach ($data as $field => $value) {
            if (empty($value)) {
                throw new LocalizedException(__('Required field %1 is missing', $field));
            }
        }
        if ($this->config->getFormType() == \Lumav\EveryPay\Model\Adminhtml\Source\FormType::FORM_TYPE_IFRAME) {
            $data['skin_name'] = $this->config->getSkinName();
        }

        return $data;
    }

    /**
     * @param int $orderId
     * @return array
     * @throws LocalizedException
     */
    public function getCitFields($orderId)
    {
        $data = [
            'api_username' => $this->config->getApiUser(),
            'account_name' => $this->config->getProcessingAccount(),
            'nonce' => $this->generateNonce(),
            'customer_url' => $this->urlBuilder->getRouteUrl('everypay/customer/landing')
        ];

        foreach ($data as $field => $value) {
            if (empty($value)) {
                throw new LocalizedException(__('Required field %1 is missing', $field));
            }
        }

        return $data;
    }

    /**
     * Generate a unique string
     *
     * @return string
     */
    private function generateNonce()
    {
        return uniqid(true);
    }
}