<?php

namespace Lumav\EveryPay\Model\Form;

class FormDataBuilder
{
    /**
     * @var \Lumav\EveryPay\Model\Config
     */
    protected $config;

    /**
     * @var \Magento\Framework\Locale\Resolver
     */
    protected $locale;

    /**
     * @var ApiData
     */
    protected $apiData;

    /**
     * @var OrderData
     */
    protected $orderData;

    /**
     * @var TokenData
     */
    protected $tokenData;

    /**
     * @var IntegrationData
     */
    protected $integrationData;

    /**
     * @param \Magento\Framework\Locale\Resolver $locale
     * @param \Lumav\EveryPay\Model\Config $config
     * @param ApiData $apiData
     * @param OrderData $orderData
     * @param TokenData $tokenData
     * @param IntegrationData $integrationData
     */
    public function __construct(
        \Magento\Framework\Locale\Resolver $locale,
        \Lumav\EveryPay\Model\Config $config,
        ApiData $apiData,
        OrderData $orderData,
        TokenData $tokenData,
        IntegrationData $integrationData
    ) {
        $this->locale = $locale;
        $this->config = $config;
        $this->apiData = $apiData;
        $this->orderData = $orderData;
        $this->tokenData = $tokenData;
        $this->integrationData = $integrationData;
    }

    /**
     * Get request data fields for payment_reference
     *
     * @param string $reference
     * @return array
     */
    public function getPaymentStatusFields($reference)
    {
        return [
            'api_username' => $this->config->getApiUser(),
            'payment_reference' => $reference
        ];
    }

    /**
     * Get request data fields for processing_accounts
     *
     * @return array
     */
    public function getProcessingAccountFields()
    {
        return [
            'api_username' => $this->config->getApiUser(),
            'account_name' => $this->config->getProcessingAccount()
        ];
    }

    /**
     * Get request data fields for oneoff payment
     *
     * @param int $orderId
     * @return array
     */
    public function getOneoffFields($orderId)
    {
        $data = array_merge(
            $this->apiData->getFields($orderId),
            $this->orderData->getFields($orderId),
            $this->tokenData->getFields(null),
            $this->integrationData->getFields()
        );
        $data['locale'] = $this->resolveLocale();
        $data['preferred_country'] = $this->getPreferredCountry();
        return $data;
    }

    /**
     * Get request data fields for CIT payment
     *
     * @param int $orderId
     * @param int $tokenId
     * @return array
     */
    public function getCitFields($orderId, $tokenId)
    {
        $tokenFields = $this->tokenData->getFields(
            $this->config->getCcTokenEnabled() ? $tokenId : null
        );

        $data = array_merge(
            $this->apiData->getCitFields($orderId),
            $this->orderData->getCitFields($orderId),
            $tokenFields,
            $this->integrationData->getFields()
        );

        return $data;
    }
    
    /**
     * Get order data fields for apple payment
     *
     * @param int $orderId
     * @return array
     */
    public function getAppleOrderData($orderId)
    {
        return $this->orderData->getAppleOrderData($orderId);
    }

    /**
     * @return string
     */
    protected function resolveLocale()
    {
        $locale = $this->locale->getLocale();
        $locale = substr($locale, 0, strpos($locale, '_'));

        if (!in_array($locale, $this->config->getSupportedLocales())) {
            $locale =  $this->config->getDefaultLocale();
        }

        return $locale;
    }

    /**
     * @return string
     */
    protected function getPreferredCountry()
    {
        $locale = $this->locale->getLocale();
        $country = substr($locale, strpos($locale, '_'));

        if (!in_array($country, $this->config->getSupportedCountries())) {
            $country = $this->config->getDefaultCountry();
        }
        return $country;
    }
}