<?php

namespace Lumav\EveryPay\Model\Form;

use Magento\Framework\Exception\LocalizedException;

class TokenData extends AbstractData
{
    const UNSCHEDULED = 'unscheduled';
    const RECURRING = 'recurring';

    /**
     * @var \Lumav\EveryPay\Api\Data\TokenRepositoryInterface
     */
    protected $tokenRepository;

    /**
     * @var \Lumav\EveryPay\Model\Token\TokenManagement
     */
    protected $tokenManager;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Lumav\EveryPay\Model\Config $config
     * @param \Lumav\EveryPay\Api\Data\TokenRepositoryInterface $tokenRepository
     * @param \Lumav\EveryPay\Model\Token\TokenManagement $tokenManager
     */
    public function __construct(
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Lumav\EveryPay\Model\Config $config,
        \Lumav\EveryPay\Api\Data\TokenRepositoryInterface $tokenRepository,
        \Lumav\EveryPay\Model\Token\TokenManagement $tokenManager,
        \Magento\Customer\Model\Session $customerSession
    ) {
        $this->tokenRepository = $tokenRepository;
        $this->tokenManager = $tokenManager;
        $this->customerSession = $customerSession;

        parent::__construct($orderRepository, $config);
    }

    /**
     * @param int $tokenId
     * @return array
     */
    public function getFields($tokenId)
    {
        if (!$tokenId || $this->isNewTokenRequest($tokenId)) {
            return $this->getRequestTokenField();
        }

        return $this->getSavedTokenFields($tokenId);
    }

    /**
     * @param int $tokenId
     * @return bool
     */
    protected function isNewTokenRequest($tokenId)
    {
        return $tokenId == \Lumav\EveryPay\Model\Config::NEW_CARD_ID;
    }

    /**
     * @return array
     */
    protected function getRequestTokenField()
    {
        return [
            'request_token' => $this->customerSession->isLoggedIn() ? true : false,
            'token_agreement' => self::UNSCHEDULED
        ];
    }

    /**
     * @param int $tokenId
     * @return array
     * @throws LocalizedException
     */
    protected function getSavedTokenFields($tokenId)
    {
        $token = $this->tokenRepository->getById($tokenId);
        $data = [
            'token_agreement' => self::UNSCHEDULED
        ];

        if ($this->tokenManager->isCustomersToken($token)) {
            $data['token'] = $token->getCcToken();
        } else {
            throw new LocalizedException(__('This card does not exist'));
        }

        return $data;
    }

}
