<?php

namespace Lumav\EveryPay\Setup;

use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;

class UpgradeData implements UpgradeDataInterface
{
    /**
     * {@inheritdoc}
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        if (version_compare($context->getVersion(), '0.0.2') < 0) {
            $connection = $setup->getConnection();            

            $tokenTable = $connection->getTableName('aedes_everypay_token');
            $newTokenTable = $connection->getTableName('lumav_everypay_token');

            //select all token values
            $this->migrateTableData($connection, $tokenTable, $newTokenTable);

            //change config values
            $coreTable = $connection->getTableName('core_config_data');
            $select = $connection->update($coreTable, ['value' => 'live'], "path LIKE 'payment/everypay/environment' AND value LIKE 'production'");
        }

        if (version_compare($context->getVersion(), '0.0.3') < 0) {
            $connection = $setup->getConnection();
            //delete nonce tables - whichever exists
            $this->deleteTable($connection, $connection->getTableName('aedes_everypay_nonce'));
            $this->deleteTable($connection, $connection->getTableName('lumav_everypay_nonce'));
        }

        if (version_compare($context->getVersion(), '0.0.4') < 0) {
            $connection = $setup->getConnection();
            $coreTable = $connection->getTableName('core_config_data');
            $connection->delete($coreTable, "path LIKE 'payment/everypay/methods' AND scope != 'default'");
        }

        $setup->endSetup();
    }

    /**
     * Migrate data from old table to new
     *
     * @param \Magento\Framework\DB\Adapter\AdapterInterface $connection
     * @param string $oldTable
     * @param string $newTable
     * @return void
     */
    protected function migrateTableData($connection, $oldTable, $newTable)
    {
        if ($connection->isTableExists($oldTable) && $connection->isTableExists($newTable)) {
            $select = $connection->select()->from($oldTable);
            $data = $connection->fetchAll($select);
            if (count($data)) {
                $connection->insertMultiple($newTable, $data);
                $connection->dropTable($oldTable);
            }

        }
    }

    /**
     * Delete table if it exists
     *
     * @param \Magento\Framework\DB\Adapter\AdapterInterface $connection
     * @param string $table
     * @return void
     */
    protected function deleteTable($connection, $table)
    {
        if ($connection->isTableExists($table)) {
            $connection->dropTable($table);
        }
    }
}