define(
    [
        'jquery',
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/model/url-builder',
        'Magento_Customer/js/model/customer',
        'mage/storage',
        'Magento_Checkout/js/model/place-order',
        'Magento_Checkout/js/model/error-processor',
        'Magento_Checkout/js/model/full-screen-loader'
    ],
    function ($, quote, urlBuilder, customer, storage, placeOrderService, errorProcessor, fullScreenLoader) {
        'use strict';

        return function (paymentData, messageContainer, tokenData, paymentMethodId) {
            var serviceUrl, payload;

            var agreementForm = $('.payment-method._active form[data-role=checkout-agreements]'),
                agreementData = agreementForm.serializeArray(),
                agreementIds = [];

            if (agreementData.length == 0) {
                agreementForm = $('.payment-method._active div[data-role=checkout-agreements] input');
                agreementData = agreementForm.serializeArray();
            }

            agreementData.forEach(function(item) {
                agreementIds.push(item.value);
            });

            if (paymentData['extension_attributes'] === undefined) {
                paymentData['extension_attributes'] = {};
            }

            paymentData['extension_attributes']['agreement_ids'] = agreementIds;


            payload = {
                cartId: quote.getQuoteId(),
                methodId: paymentMethodId,
                billingAddress: quote.billingAddress(),
                paymentMethod: paymentData
            };

            if (customer.isLoggedIn()) {
                serviceUrl = urlBuilder.createUrl('/everypay/payment-adapter/mine', {});
                payload.tokenId = tokenData.tokenId ? tokenData.tokenId : 0;
            } else {
                serviceUrl = urlBuilder.createUrl('/everypay/payment-adapter/:quoteId', {
                    quoteId: quote.getQuoteId()
                });
                payload.email = quote.guestEmail;
            }

            return storage.post(
                serviceUrl,
                JSON.stringify(payload)
            ).fail(
                function (response) {
                    errorProcessor.process(response, messageContainer);
                    fullScreenLoader.stopLoader();
                }
            );
        };
    }
);
