define([
    'ko',
    'uiComponent',
    'Lumav_EveryPay/js/model/method'
], function(ko, Component, method) {
    "use strict";

    return Component.extend({
        selectedMethod: ko.observable(null),
        selectedCountry: ko.observable(window.checkoutConfig.payment.everypay.defaultCountry),
        methodCollection: [],
        cards: null,
        banks: null,
        alternatives: null,
        applePayAvailable: window.checkoutConfig.payment.everypay.applepay_available,
        googlePayAvailable: window.checkoutConfig.payment.everypay.googlepay_available,
        
        initialize: function() {
            this._super();
            var methods = method.prototype.fromData(window.checkoutConfig.payment.everypay.methods);
            this.setCollection(methods);
            this.selectCountry(window.checkoutConfig.payment.everypay.defaultCountry);
        },
        setCollection: function(collection) {
            this.methodCollection = collection;
        },
        getCollection: function() {
            return this.methodCollection;
        },
        getMethods: function(type) {
            switch (type) {
                case this.getCardsCode():
                    return this.getCards();
                    break;
                case this.getBanksCode():
                    return this.getBanks();
                    break;
                case this.getAlternativeCode():
                    return this.getAlternatives();
                    break;
                default:
                    return [];
            }
        },
        getCards: function() {
            if (!this.cards) {
                var self = this;
                this.cards = this.getCollection().filter(function(method) {
                    return method.getType() === self.getCardsCode();
                });
            }
            return this.cards;
        },
        getBanks: function() {
            if (!this.banks) {
                var self = this;
                this.banks = this.getCollection().filter(function(method) {
                    return method.getType() === self.getBanksCode();
                });
            }
            return this.banks;
        },
        getAlternatives: function() {
            if (!this.alternatives) {
                var self = this;
                this.alternatives = this.getCollection().filter(function(method) {
                    return method.getType() === self.getAlternativeCode();
                });
            }
            return this.alternatives;
        },
        selectMethod: function(method) {
            var self = this;
            return function() {
                self.selectedMethod(method);
                return true;
            }
        },
        getSelectedMethod: function() {
            return this.selectedMethod();
        },
        isCardSelected: function() {
            return this.selectedMethod() ? this.selectedMethod().getType() === 'cards' : false;
        },
        isAppleSelected: function() {
            return this.selectedMethod() ? this.selectedMethod().getType() === 'apple' : false;
        },
        isGoogleSelected: function() {
            return this.selectedMethod() ? this.selectedMethod().getType() === 'google' : false;
        },
        isMethodSelected: function(method) {
            return this.selectedMethod() ? this.selectedMethod().getId() === method.getId() : false;
        },
        isCardsVisible: function() {
            return this.isTypeVisible(this.getCardsCode());
        },
        isBanksVisible: function() {
            return this.isTypeVisible(this.getBanksCode());
        },
        isAlternativesVisible: function() {
            return this.isTypeVisible(this.getAlternativeCode());
        },
        isAppleVisible: function() {
            if(window.ApplePaySession) {
                if(ApplePaySession.canMakePayments()) {
                    return this.applePayAvailable;
                }
            }
            return false;
        },
        isGoogleVisible: function() {
            return this.googlePayAvailable;
        },
        isTypeVisible: function(type) {
            return this.getVisibleMethods(type).length > 0;
        },
        getVisibleMethods: function(type) {
            var collection = this.getMethods(type);
            if (collection.length > 0) {
                return collection.filter(function(method) {
                    return method.isVisible();
                })
            }
            return [];
        },
        selectCountry: function(country) {
            this.getCollection().forEach(function(method) {
                method.setVisibility(
                    method.getCountry() == null ||
                    method.getCountry() == 'N/A' ||
                    method.getCountry() === country
                );
            });
        },
        getCardsCode: function() {
            return 'cards';
        },
        getBanksCode: function() {
            return 'banks';
        },
        getAlternativeCode: function() {
            return 'alternative';
        },
        getAppleCode: function() {
            return 'apple';
        },
        getGoogleCode: function() {
            return 'google';
        },
        getAppleButtonStyle: function() {
            return window.checkoutConfig.payment.everypay.appleButton
        },
        getGoogleButtonStyle: function() {
            return window.checkoutConfig.payment.everypay.googleButton
        }
    });
});
