define([
    'ko',
    'uiComponent',
    'Magento_Checkout/js/model/quote',
    'Lumav_EveryPay/js/model/method-manager',
    'mage/translate'
], function(ko, Component, quote, methodManager, $t) {
    "use strict";

    return Component.extend({
        activeType: ko.observable(null),
        methodManager: methodManager(),
        setCacheItem: sessionStorage.setItem.bind(sessionStorage),
        getCacheItem: sessionStorage.getItem.bind(sessionStorage),
        
        initialize: function() {
            this._super();
            var fromCache = this.getCacheItem('everypay-selected-type');
            if (typeof fromCache !== 'undefined' && fromCache) {
                this.setActiveType(fromCache);
            }
        },
        isCardsSelected: function() {
            return this.isTypeSelected(this.methodManager.getCardsCode());
        },
        isBanksSelected: function() {
            return this.isTypeSelected(this.methodManager.getBanksCode());
        },
        isAlternativeSelected: function() {
            return this.isTypeSelected(this.methodManager.getAlternativeCode());
        },
        isAppleSelected: function() {
            return this.isTypeSelected(this.methodManager.getAppleCode());
        },
        isGoogleSelected: function() {
            return this.isTypeSelected(this.methodManager.getGoogleCode());
        },
        isTypeSelected: function(type) {
            var self = this;
            return ko.computed(function () {
                if (quote.paymentMethod()) {
                    return (quote.paymentMethod().method === 'everypay' && self.getActiveType() === type)
                        ? 'everypay' + type
                        : null;
                }
                return null;
            });
        },
        setActiveType: function(type) {
            var visibleMethods = this.methodManager.getVisibleMethods(type);
            if (visibleMethods.length === 1) {
                this.methodManager.selectMethod(visibleMethods[0])();
            }
            this.setCacheItem('everypay-selected-type', type);
            this.activeType(type);
        },
        getActiveType: function() {
            return this.activeType();
        },
        getCardsTitle: function() {
            return window.checkoutConfig.payment.everypay.titles.cards;
        },
        getBanksTitle: function() {
            return window.checkoutConfig.payment.everypay.titles.banks;
        },
        getAlternativeTitle: function() {
            return window.checkoutConfig.payment.everypay.titles.alternative;
        },
        getAppleTitle: function() {
            return window.checkoutConfig.payment.everypay.titles.apple;
        },
        getGoogleTitle: function() {
            return window.checkoutConfig.payment.everypay.titles.google;
        },
        getMethodAreaTitle: function() {
            return $t('Available Methods')
        }
    });
});
